/* -----------------------------------------------------------------------
   See COPYRIGHT.TXT and LICENSE.TXT for copyright and license information
   ----------------------------------------------------------------------- */
#ifndef __plm_config_h__
#define __plm_config_h__

/* ITK 3.20 is missing this */
#if defined __cplusplus
#include <stddef.h>
#endif

/* Architecture attributes */
#define MACHINE_IS_32_BIT 1
/* #undef MACHINE_IS_64_BIT */

/* Major OSes */
/* #undef WIN32 */
#define UNIX  1
/* #undef APPLE */

/* Compiler attributes */
/* #undef FORTRAN_COMPILER_FOUND */
#define HAVE_SYS_STAT_H 1
#define HAVE_STDINT_H 1
#define HAVE_SYS_STAT_H 1
#define OPENMP_FOUND 1
/* #undef SSE2_FOUND */
#define CMAKE_SIZEOF_UINT 4
#define CMAKE_SIZEOF_ULONG 4
#define CMAKE_SIZEOF_SIZE_T 4
#define PLM_BIG_ENDIAN 1

/* Plastimatch version attributes */
/* #undef PLASTIMATCH_HAVE_BUILD_NUMBER */
#define PLASTIMATCH_BUILD_NUMBER "EXPERIMENTAL"
#define PLM_DEFAULT_VERSION_STRING "1.5.16"
#define PLM_RELEASE_VERSION_STRING "1.5.16 (4659)"
/* #undef PLM_CONFIG_VERSION_STRING */

/* External libraries */
/* #undef CUDA_FOUND */
#define DCMTK_FOUND 1
#define DCMTK_HAVE_CFUNIX_H 1
/* #undef DLIB_FOUND */
#define FFTW_FOUND 1
#define GDCM_FOUND 1
/* #undef GDCM_VERSION_1 */
#define GDCM_VERSION_2 1
#define LIBLBFGS_FOUND 1
/* #undef NLOPT_FOUND */
/* #undef OPENCL_FOUND */
/* #undef PANTHEIOS_FOUND */
#define SPECFUN_FOUND 1
#define HAVE_GETOPT_LONG 1
/* #undef READLINE_FOUND */
/* #undef QT4_FOUND */

/* User options */
/* #undef PLM_BUILD_SHARED_LIBS */
#define PLM_CONFIG_USE_SS_IMAGE_VEC 1
/* #undef PLM_CONFIG_ALT_DCOS */
/* #undef PLM_CONFIG_KEYHOLIZE */
#define PLM_CONFIG_PREFER_DCMTK 1

/* Needed for dynamic loading */
/* #undef PLM_USE_GPU_PLUGINS */

/* Did we find fortran? -- always true now. */
#define FORTRAN_FOUND 1

/* Dicom attributes */
#define PLM_DCM_USE_DCMTK 1

/* ITK attributes */
#define ITK_FOUND 1
#define ITK_USE_SYSTEM_GDCM 1
#if (ITK_FOUND && !PLM_CUDA_COMPILE)
#include "itkConfigure.h"
//#if !defined (ITK_LEGACY_REMOVE)
//#define ITK_LEGACY_SILENT 1 /* Silence MSVC+ITK compiler warnings */
//#endif
#if (ITK_VERSION_MAJOR >= 3) && (ITK_VERSION_MINOR >= 10) && defined (ITK_USE_ORIENTED_IMAGE_DIRECTION)
#define PLM_ITK_ORIENTED_IMAGES 1
#endif
#endif

/* Make Microsoft compiler less whiny */
#if _MSC_VER >= 1400
/* 4244 warning convert double to float, etc. */
/* 4251 dll export of std c++ types
   http://msdn.microsoft.com/en-us/library/esew7y1w.aspx */
/* 4305 warning convert double to float, etc. */
/* 4819 warnings generated by itk headers */
/* 4800 warning convert int to bool */
/* 4996 warnings for using functions defined as deprecated */
#pragma warning( disable : 4244 4251 4305 4800 4819 4996 )
#endif

/* Make GCC compiler less whiny */
#if (__GNUC__ >= 4) && (__GNUC_MINOR__ >= 2) && !defined(NVCC)
/* -Wwrite-strings warnings generated when passing literal strings as 
   (non-const?) function parameters */
#pragma GCC diagnostic ignored "-Wwrite-strings"
/* -Wdepricated warnings generated by itk headers */
#pragma GCC diagnostic ignored "-Wdeprecated"
/* -Wno-sign-conversion warnings are generated when comparing signed 
   and unsigned integers -- these should be revisited occasionally 
   to check for actual bugs. */
/* Unfortunately, the below doesn't actually work.  We still get warnings. */
/**** #pragma GCC diagnostic ignored "-Wsign-conversion" */
/* -Wmissing-braces warnings are generated when nested braces aren't used
   for initialization of multi-dimensional array */
#pragma GCC diagnostic ignored "-Wmissing-braces"
#endif
/* -Wunused-local-typedefs is emitted by older versions of ITK */
#if (__GNUC__ >= 4) && (__GNUC_MINOR__ >= 8) && !defined(NVCC)
# if (ITK_FOUND && (ITK_VERSION_MAJOR == 3))
# pragma GCC diagnostic ignored "-Wunused-local-typedefs"
# endif
#endif

/* Make clang compiler less whiny */
#if defined (__clang__)
/* -Wgnu ignores GNU extensions, which is needed to suppress warnings 
   caused by non-compliant vxl header files */
#pragma clang diagnostic ignored "-Wgnu"
/* -Wtautological-compare warnings are generated by comparison expressions 
   which are always true or always false -- usually caused by 
   template expansions */
#pragma clang diagnostic ignored "-Wtautological-compare"
#endif

#if _MSC_VER < 1700
#define inline __inline
#endif

#if defined(__BORLANDC__) || defined(_MSC_VER)
#define snprintf _snprintf
#ifndef va_copy
#define va_copy(d,s) ((d) = (s))
#endif
#endif


/* JAS 2012.04.24
 * Plastimatch 2.0 method for demangling symbols */
#ifdef __cplusplus
#define EXTERNC extern "C"
#else
#define EXTERNC
#endif


/* JAS 2011.02.07
 * GPU Plugins are not considered "shared libs"
 * otherwise Win32 build will break when not using "shared libs" */
#if ((defined(_WIN32) || defined(WIN32)) && (defined (PLM_USE_GPU_PLUGINS)))
#ifdef plmcuda_EXPORTS
 #define plmcuda_EXPORT(f, ...)                                     \
    __declspec(dllexport)                                          \
    f (__VA_ARGS__); typedef f##_t(__VA_ARGS__); 
 #else
 #define plmcuda_EXPORT(f, ...)                                     \
    __declspec(dllimport)                                          \
    f (__VA_ARGS__); typedef f##_t(__VA_ARGS__); 
#endif

#ifdef plmopencl_EXPORTS
 #define plmopencl_EXPORT(f, ...)                                   \
    __declspec(dllexport)                                          \
    f (__VA_ARGS__); typedef f##_t(__VA_ARGS__); 
#else
 #define plmopencl_EXPORT(f, ...)                                   \
    __declspec(dllimport)                                          \
    f (__VA_ARGS__); typedef f##_t(__VA_ARGS__); 
#endif

#else
#define plmcuda_EXPORT(f, ...)                                     \
    f (__VA_ARGS__); typedef f##_t(__VA_ARGS__); 
#define plmopencl_EXPORT(f, ...)                                   \
    f (__VA_ARGS__); typedef f##_t(__VA_ARGS__); 
#endif

#endif /* __plm_config_h__ */
